@@ -33,6 +33,9 @@ static snmp_device_id_t snmp_device_table[] = {
 	{ ".1.3.6.1.4.1.17373.3.1.1.0" ,  "aphel_genesisII", ".1.3.6.1.4.1.17373"},
 	{ ".1.3.6.1.4.1.534.6.6.6.1.1.12.0" ,  "aphel_revelation", ".1.3.6.1.4.1.534.6.6.6"},
 	{ ".1.3.6.1.4.1.534.6.6.7.1.2.1.2.0" ,  "eaton_epdu", ".1.3.6.1.4.1.534.6.6.7"},
+	{ "" ,  "pulizzi_monitored", NULL},
+	{ ".1.3.6.1.4.1.20677.1" ,  "pulizzi_switched1", ".1.3.6.1.4.1.20677.1"},
+	{ ".1.3.6.1.4.1.20677.1" ,  "pulizzi_switched2", ".1.3.6.1.4.1.20677.2"},
 	{ "1.3.6.1.4.1.534.1.1.2.0" ,  "pw", ".1.3.6.1.4.1.534.1"},
 	{ ".1.3.6.1.4.1.705.1.1.1.0" ,  "mge", ".1.3.6.1.4.1.705.1"},
 	{ ".1.3.6.1.4.1.318.1.1.1.1.1.1.0" ,  "apcc", NULL},
