@@ -36,6 +36,7 @@
 #include <ltdl.h>
 
 /* dynamic link library stuff */
+static char * libname = "libavahi-client";
 static lt_dlhandle dl_handle = NULL;
 static const char *dl_error = NULL;
 
@@ -98,7 +99,7 @@ int nutscan_load_avahi_library()
                 return 0;
         }
 
-        dl_handle = lt_dlopenext("libavahi-client");
+        dl_handle = lt_dlopenext(libname);
         if (!dl_handle) {
                 dl_error = lt_dlerror();
                 goto err;
@@ -196,8 +197,10 @@ int nutscan_load_avahi_library()
 
         return 1;
 err:
-        fprintf(stderr, "%s\n", dl_error);
+        fprintf(stderr, "Cannot load AVAHI library (%s) : %s. AVAHI search disabled.\n", libname, dl_error);
+
         dl_handle = (void *)1;
+	lt_dlexit();
         return 0;
 }
 /* end of dynamic link library stuff */
